/*
 *   FreeAdhocUDF - copyright (c) 2004-2008 adhoc dataservice GmbH
 *   Christoph Theuring / Peter Mandrella / Georg Horn
 *   (bug-)reports only to udf@adhoc-data.de
 *   based on FreeUDFLibC - copyright (c) 1999 Gregory Deatz
 *
 *   Declarations script for rFunc UDFs with FreeAdhocUDF EntryPoints
 *
 *   Declarations with function-names, input and output exactly like rFunc
 *   RETURN-mechanism, entrypoint and module-name like FreeAdhocUDF
 *
 *   we recommended to use the FreeAdhocUDF declarations
 *
 *   Notice: to use with dialect 3 (timestamp-field is TIMESTAMP)
 *
 */

SET SQL DIALECT 3;

/******************************************************************************/
/****                 User Defined Functions: Date and TIME                ****/
/******************************************************************************/

DECLARE EXTERNAL FUNCTION EXTRACTYEAR
   TIMESTAMP
   RETURNS INTEGER FREE_IT
  ENTRY_POINT 'year'  MODULE_NAME 'FreeAdhocUDF';

DECLARE EXTERNAL FUNCTION EXTRACTMONTH
   TIMESTAMP
   RETURNS INTEGER FREE_IT
  ENTRY_POINT 'month'  MODULE_NAME 'FreeAdhocUDF';

DECLARE EXTERNAL FUNCTION EXTRACTDAY
   TIMESTAMP
   RETURNS INTEGER FREE_IT
  ENTRY_POINT 'dayofmonth'  MODULE_NAME 'FreeAdhocUDF';

DECLARE EXTERNAL FUNCTION EXTRACTYEARDAY
   TIMESTAMP
   RETURNS INTEGER FREE_IT
  ENTRY_POINT 'r_yearday'  MODULE_NAME 'FreeAdhocUDF';

DECLARE EXTERNAL FUNCTION EXTRACTWEEKDAY
   TIMESTAMP
   RETURNS INTEGER FREE_IT
  ENTRY_POINT 'r_weekday'  MODULE_NAME 'FreeAdhocUDF';

DECLARE EXTERNAL FUNCTION EXTRACTHOUR
   TIMESTAMP
   RETURNS INTEGER FREE_IT
  ENTRY_POINT 'hour'  MODULE_NAME 'FreeAdhocUDF';

DECLARE EXTERNAL FUNCTION EXTRACTMINUTE
   TIMESTAMP
   RETURNS INTEGER FREE_IT
  ENTRY_POINT 'minute'  MODULE_NAME 'FreeAdhocUDF';

DECLARE EXTERNAL FUNCTION EXTRACTSECOND
   TIMESTAMP
   RETURNS INTEGER FREE_IT
  ENTRY_POINT 'second'  MODULE_NAME 'FreeAdhocUDF';

DECLARE EXTERNAL FUNCTION ISLEAPYEAR
   INTEGER
   RETURNS INTEGER FREE_IT
  ENTRY_POINT 'r_isleapyear'  MODULE_NAME 'FreeAdhocUDF';

DECLARE EXTERNAL FUNCTION EXTRACTMILLISECOND
   TIMESTAMP
   RETURNS INTEGER FREE_IT
  ENTRY_POINT 'millisecond'  MODULE_NAME 'FreeAdhocUDF';

DECLARE EXTERNAL FUNCTION QUARTER
   TIMESTAMP
   RETURNS INTEGER FREE_IT
  ENTRY_POINT 'quarter'  MODULE_NAME 'FreeAdhocUDF';

DECLARE EXTERNAL FUNCTION DATETOSTR
   TIMESTAMP, CSTRING(255)
   RETURNS CSTRING(255) FREE_IT
  ENTRY_POINT 'datetostr'  MODULE_NAME 'FreeAdhocUDF';

DECLARE EXTERNAL FUNCTION ENCODEDATE
   INTEGER, INTEGER, INTEGER
   RETURNS TIMESTAMP FREE_IT
  ENTRY_POINT 'r_encodedate'  MODULE_NAME 'FreeAdhocUDF';

DECLARE EXTERNAL FUNCTION ENCODEDATETIME
   INTEGER, INTEGER, INTEGER, INTEGER, INTEGER, INTEGER
   RETURNS TIMESTAMP FREE_IT
  ENTRY_POINT 'r_encodedatetime'  MODULE_NAME 'FreeAdhocUDF';

DECLARE EXTERNAL FUNCTION EXTRACTDATE
   TIMESTAMP
   RETURNS TIMESTAMP
  ENTRY_POINT 'striptime'  MODULE_NAME 'FreeAdhocUDF';

DECLARE EXTERNAL FUNCTION EXTRACTTIME
   TIMESTAMP
   RETURNS TIMESTAMP
  ENTRY_POINT 'stripdate'  MODULE_NAME 'FreeAdhocUDF';

DECLARE EXTERNAL FUNCTION DOW
   TIMESTAMP
   RETURNS INTEGER FREE_IT
  ENTRY_POINT 'r_dow'  MODULE_NAME 'FreeAdhocUDF';

DECLARE EXTERNAL FUNCTION DAYPERMONTH
   INTEGER, INTEGER
   RETURNS INTEGER FREE_IT
  ENTRY_POINT 'daysofmonth'  MODULE_NAME 'FreeAdhocUDF';

DECLARE EXTERNAL FUNCTION FIRSTDAYMONTH
   TIMESTAMP
   RETURNS TIMESTAMP
  ENTRY_POINT 'firstdaymonth'  MODULE_NAME 'FreeAdhocUDF';

DECLARE EXTERNAL FUNCTION LASTDAYMONTH
   TIMESTAMP
   RETURNS TIMESTAMP
  ENTRY_POINT 'lastdaymonth'  MODULE_NAME 'FreeAdhocUDF';

DECLARE EXTERNAL FUNCTION DAYSBETWEEN
   TIMESTAMP, TIMESTAMP
   RETURNS INTEGER FREE_IT
  ENTRY_POINT 'ageindays'  MODULE_NAME 'FreeAdhocUDF';

DECLARE EXTERNAL FUNCTION INCDATE
   TIMESTAMP, INTEGER, INTEGER, INTEGER
   RETURNS TIMESTAMP
  ENTRY_POINT 'incdate'  MODULE_NAME 'FreeAdhocUDF';

DECLARE EXTERNAL FUNCTION INCDATETIME
   TIMESTAMP, INTEGER, INTEGER, INTEGER, INTEGER, INTEGER, INTEGER
   RETURNS TIMESTAMP
  ENTRY_POINT 'incdatetime'  MODULE_NAME 'FreeAdhocUDF';

DECLARE EXTERNAL FUNCTION MAXDATE
   TIMESTAMP, TIMESTAMP
   RETURNS TIMESTAMP
  ENTRY_POINT 'maxdate'  MODULE_NAME 'FreeAdhocUDF';

DECLARE EXTERNAL FUNCTION MINDATE
   TIMESTAMP, TIMESTAMP
   RETURNS TIMESTAMP
  ENTRY_POINT 'mindate'  MODULE_NAME 'FreeAdhocUDF';

DECLARE EXTERNAL FUNCTION TIMETODOUBLE
   TIMESTAMP
   RETURNS DOUBLE PRECISION FREE_IT
  ENTRY_POINT 'timetodouble'  MODULE_NAME 'FreeAdhocUDF';

DECLARE EXTERNAL FUNCTION DATETODOUBLE
   TIMESTAMP
   RETURNS DOUBLE PRECISION FREE_IT
  ENTRY_POINT 'datetodouble'  MODULE_NAME 'FreeAdhocUDF';

DECLARE EXTERNAL FUNCTION DOUBLETODATE
   DOUBLE PRECISION
   RETURNS TIMESTAMP FREE_IT
  ENTRY_POINT 'doubletodate'  MODULE_NAME 'FreeAdhocUDF';

DECLARE EXTERNAL FUNCTION DOUBLETOTIME
   DOUBLE PRECISION
   RETURNS TIMESTAMP FREE_IT
  ENTRY_POINT 'doubletotime'  MODULE_NAME 'FreeAdhocUDF';

DECLARE EXTERNAL FUNCTION DTEQUAL
   TIMESTAMP, TIMESTAMP
   RETURNS INTEGER FREE_IT
  ENTRY_POINT 'equaldatetime'  MODULE_NAME 'FreeAdhocUDF';

/******************************************************************************/
/****            User Defined Functions - numeric/math functions           ****/
/******************************************************************************/

DECLARE EXTERNAL FUNCTION ABS
   DOUBLE PRECISION
   RETURNS DOUBLE PRECISION FREE_IT
  ENTRY_POINT 'doubleabs'  MODULE_NAME 'FreeAdhocUDF';

DECLARE EXTERNAL FUNCTION MAXNUM
   DOUBLE PRECISION, DOUBLE PRECISION
   RETURNS DOUBLE PRECISION FREE_IT
  ENTRY_POINT 'maxnum'  MODULE_NAME 'FreeAdhocUDF';

DECLARE EXTERNAL FUNCTION MINNUM
   DOUBLE PRECISION, DOUBLE PRECISION
   RETURNS DOUBLE PRECISION FREE_IT
  ENTRY_POINT 'minnum'  MODULE_NAME 'FreeAdhocUDF';

DECLARE EXTERNAL FUNCTION CEIL
   DOUBLE PRECISION
   RETURNS DOUBLE PRECISION FREE_IT
  ENTRY_POINT 'f_ceiling'  MODULE_NAME 'FreeAdhocUDF';

DECLARE EXTERNAL FUNCTION FLOOR
   DOUBLE PRECISION
   RETURNS DOUBLE PRECISION FREE_IT
  ENTRY_POINT 'f_floor'  MODULE_NAME 'FreeAdhocUDF';

DECLARE EXTERNAL FUNCTION POWER
   DOUBLE PRECISION, DOUBLE PRECISION
   RETURNS DOUBLE PRECISION FREE_IT
  ENTRY_POINT 'f_power'  MODULE_NAME 'FreeAdhocUDF';

DECLARE EXTERNAL FUNCTION ROUND
   DOUBLE PRECISION, INTEGER
   RETURNS DOUBLE PRECISION FREE_IT
  ENTRY_POINT 'zahlrunden'  MODULE_NAME 'FreeAdhocUDF';

DECLARE EXTERNAL FUNCTION SOFTROUND
   DOUBLE PRECISION, INTEGER
   RETURNS DOUBLE PRECISION FREE_IT
  ENTRY_POINT 'softround'  MODULE_NAME 'FreeAdhocUDF';

DECLARE EXTERNAL FUNCTION DIV
   INTEGER, INTEGER
   RETURNS INTEGER FREE_IT
  ENTRY_POINT 'r_div'  MODULE_NAME 'FreeAdhocUDF';

DECLARE EXTERNAL FUNCTION MOD
   INTEGER, INTEGER
   RETURNS INTEGER FREE_IT
  ENTRY_POINT 'modulo'  MODULE_NAME 'FreeAdhocUDF';

DECLARE EXTERNAL FUNCTION Z
   DOUBLE PRECISION
   RETURNS DOUBLE PRECISION BY VALUE
  ENTRY_POINT 'r_z'  MODULE_NAME 'FreeAdhocUDF';

DECLARE EXTERNAL FUNCTION DZERO
   DOUBLE PRECISION, DOUBLE PRECISION, DOUBLE PRECISION
   RETURNS DOUBLE PRECISION BY VALUE
  ENTRY_POINT 'r_dividezero'  MODULE_NAME 'FreeAdhocUDF';

DECLARE EXTERNAL FUNCTION GETRANDOM
   INTEGER
   RETURNS INTEGER FREE_IT
  ENTRY_POINT 'r_getrandom'  MODULE_NAME 'FreeAdhocUDF';

DECLARE EXTERNAL FUNCTION BITAND
   INTEGER, INTEGER
   RETURNS INTEGER FREE_IT
  ENTRY_POINT 'f_bin_and'  MODULE_NAME 'FreeAdhocUDF';

DECLARE EXTERNAL FUNCTION BITOR
   INTEGER, INTEGER
   RETURNS INTEGER FREE_IT
  ENTRY_POINT 'f_bin_or'  MODULE_NAME 'FreeAdhocUDF';

DECLARE EXTERNAL FUNCTION BITXOR
   INTEGER, INTEGER
   RETURNS INTEGER FREE_IT
  ENTRY_POINT 'f_bin_xor'  MODULE_NAME 'FreeAdhocUDF';

DECLARE EXTERNAL FUNCTION BITNOT
   INTEGER
   RETURNS INTEGER FREE_IT
  ENTRY_POINT 'f_bin_not'  MODULE_NAME 'FreeAdhocUDF';

DECLARE EXTERNAL FUNCTION SIGN
   DOUBLE PRECISION
   RETURNS SMALLINT FREE_IT
  ENTRY_POINT 'f_sign'  MODULE_NAME 'FreeAdhocUDF';

DECLARE EXTERNAL FUNCTION ACOS
   DOUBLE PRECISION
   RETURNS DOUBLE PRECISION FREE_IT
  ENTRY_POINT 'arccos_rad'  MODULE_NAME 'FreeAdhocUDF';

DECLARE EXTERNAL FUNCTION ASIN
   DOUBLE PRECISION
   RETURNS DOUBLE PRECISION FREE_IT
  ENTRY_POINT 'arcsin_rad'  MODULE_NAME 'FreeAdhocUDF';

DECLARE EXTERNAL FUNCTION ATAN
   DOUBLE PRECISION
   RETURNS DOUBLE PRECISION FREE_IT
  ENTRY_POINT 'arctan_rad'  MODULE_NAME 'FreeAdhocUDF';

DECLARE EXTERNAL FUNCTION ATAN2
   DOUBLE PRECISION, DOUBLE PRECISION
   RETURNS DOUBLE PRECISION FREE_IT
  ENTRY_POINT 'arctan2_rad'  MODULE_NAME 'FreeAdhocUDF';

DECLARE EXTERNAL FUNCTION COS
   DOUBLE PRECISION
   RETURNS DOUBLE PRECISION FREE_IT
  ENTRY_POINT 'cos_rad'  MODULE_NAME 'FreeAdhocUDF';

DECLARE EXTERNAL FUNCTION COSH
   DOUBLE PRECISION
   RETURNS DOUBLE PRECISION FREE_IT
  ENTRY_POINT 'coshyp'  MODULE_NAME 'FreeAdhocUDF';

DECLARE EXTERNAL FUNCTION EXP
   DOUBLE PRECISION
   RETURNS DOUBLE PRECISION FREE_IT
  ENTRY_POINT 'f_exp'  MODULE_NAME 'FreeAdhocUDF';

DECLARE EXTERNAL FUNCTION LOG
   DOUBLE PRECISION, DOUBLE PRECISION
   RETURNS DOUBLE PRECISION FREE_IT
  ENTRY_POINT 'f_log'  MODULE_NAME 'FreeAdhocUDF';

DECLARE EXTERNAL FUNCTION LN
   DOUBLE PRECISION
   RETURNS DOUBLE PRECISION FREE_IT
  ENTRY_POINT 'f_ln'  MODULE_NAME 'FreeAdhocUDF';

DECLARE EXTERNAL FUNCTION LOG10
   DOUBLE PRECISION
   RETURNS DOUBLE PRECISION FREE_IT
  ENTRY_POINT 'f_log10'  MODULE_NAME 'FreeAdhocUDF';

DECLARE EXTERNAL FUNCTION SIN
   DOUBLE PRECISION
   RETURNS DOUBLE PRECISION FREE_IT
  ENTRY_POINT 'sin_rad'  MODULE_NAME 'FreeAdhocUDF';

DECLARE EXTERNAL FUNCTION SINH
   DOUBLE PRECISION
   RETURNS DOUBLE PRECISION FREE_IT
  ENTRY_POINT 'sinhyp'  MODULE_NAME 'FreeAdhocUDF';

DECLARE EXTERNAL FUNCTION SQRT
   DOUBLE PRECISION
   RETURNS DOUBLE PRECISION FREE_IT
  ENTRY_POINT 'f_sqrt'  MODULE_NAME 'FreeAdhocUDF';

DECLARE EXTERNAL FUNCTION TAN
   DOUBLE PRECISION
   RETURNS DOUBLE PRECISION FREE_IT
  ENTRY_POINT 'tan_rad'  MODULE_NAME 'FreeAdhocUDF';

DECLARE EXTERNAL FUNCTION TANH
   DOUBLE PRECISION
   RETURNS DOUBLE PRECISION FREE_IT
  ENTRY_POINT 'tanhyp'  MODULE_NAME 'FreeAdhocUDF';

DECLARE EXTERNAL FUNCTION IEQUAL
   INTEGER, INTEGER
   RETURNS INTEGER FREE_IT
  ENTRY_POINT 'equalinteger'  MODULE_NAME 'FreeAdhocUDF';

DECLARE EXTERNAL FUNCTION DEQUAL
   DOUBLE PRECISION, DOUBLE PRECISION
   RETURNS INTEGER FREE_IT
  ENTRY_POINT 'equalfloat'  MODULE_NAME 'FreeAdhocUDF';


/******************************************************************************/
/****              User Defined Functions - constant functions             ****/
/******************************************************************************/

DECLARE EXTERNAL FUNCTION E
   RETURNS DOUBLE PRECISION BY VALUE
  ENTRY_POINT 'euler'  MODULE_NAME 'FreeAdhocUDF';

DECLARE EXTERNAL FUNCTION PI
   RETURNS DOUBLE PRECISION BY VALUE
  ENTRY_POINT 'pi'  MODULE_NAME 'FreeAdhocUDF';

/******************************************************************************/
/****              User Defined Functions - string functions               ****/
/****                            NOT COMPLETE                              ****/
/******************************************************************************/

DECLARE EXTERNAL FUNCTION CHR
   SMALLINT
   RETURNS CSTRING(1) FREE_IT
  ENTRY_POINT 'character'  MODULE_NAME 'FreeAdhocUDF';

DECLARE EXTERNAL FUNCTION ORD
   CSTRING(1)
   RETURNS SMALLINT FREE_IT
  ENTRY_POINT 'ord'  MODULE_NAME 'FreeAdhocUDF';

DECLARE EXTERNAL FUNCTION LTRIM
   CSTRING(255)
   RETURNS CSTRING(255)
  ENTRY_POINT 'ltrim'  MODULE_NAME 'FreeAdhocUDF';

DECLARE EXTERNAL FUNCTION LONGLTRIM
   CSTRING(16383)
   RETURNS CSTRING(16383)
  ENTRY_POINT 'ltrim'  MODULE_NAME 'FreeAdhocUDF';

DECLARE EXTERNAL FUNCTION RTRIM
   CSTRING(255)
   RETURNS CSTRING(255)
  ENTRY_POINT 'rtrim'  MODULE_NAME 'FreeAdhocUDF';

DECLARE EXTERNAL FUNCTION LONGRTRIM
   CSTRING(16383)
   RETURNS CSTRING(16383)
  ENTRY_POINT 'rtrim'  MODULE_NAME 'FreeAdhocUDF';

DECLARE EXTERNAL FUNCTION TRIM
   CSTRING(255)
   RETURNS CSTRING(255)
  ENTRY_POINT 'lrtrim'  MODULE_NAME 'FreeAdhocUDF';

DECLARE EXTERNAL FUNCTION STRTRIM
   CSTRING(255)
   RETURNS CSTRING(255)
  ENTRY_POINT 'lrtrim'  MODULE_NAME 'FreeAdhocUDF';

DECLARE EXTERNAL FUNCTION LONGTRIM
   CSTRING(16383)
   RETURNS CSTRING(16383)
  ENTRY_POINT 'lrtrim'  MODULE_NAME 'FreeAdhocUDF';

DECLARE EXTERNAL FUNCTION SUBSTR
   CSTRING(255), INTEGER, INTEGER
   RETURNS CSTRING(255)
  ENTRY_POINT 'mid1'  MODULE_NAME 'FreeAdhocUDF';

DECLARE EXTERNAL FUNCTION LONGSUBSTR
   CSTRING(16383), INTEGER, INTEGER
   RETURNS CSTRING(16383)
  ENTRY_POINT 'mid1'  MODULE_NAME 'FreeAdhocUDF';

DECLARE EXTERNAL FUNCTION STRREPLACE
   CSTRING(255), CSTRING(255), CSTRING(255)
   RETURNS CSTRING(255) FREE_IT
  ENTRY_POINT 'replace'  MODULE_NAME 'FreeAdhocUDF';

DECLARE EXTERNAL FUNCTION LONGSTRREPLACE
   CSTRING(16383), CSTRING(16383), CSTRING(16383)
   RETURNS CSTRING(16383) FREE_IT
  ENTRY_POINT 'replace'  MODULE_NAME 'FreeAdhocUDF';

DECLARE EXTERNAL FUNCTION STRLEN
   CSTRING(16383)
   RETURNS INTEGER FREE_IT
  ENTRY_POINT 'stringlength'  MODULE_NAME 'FreeAdhocUDF';

DECLARE EXTERNAL FUNCTION CEQUAL
   CSTRING(16383), CSTRING(16383)
   RETURNS INTEGER FREE_IT
  ENTRY_POINT 'equalstring'  MODULE_NAME 'FreeAdhocUDF';

DECLARE EXTERNAL FUNCTION RUPPER
   CSTRING(255)
   RETURNS CSTRING(255)
  ENTRY_POINT 'upper'  MODULE_NAME 'FreeAdhocUDF';

DECLARE EXTERNAL FUNCTION LONGRUPPER
   CSTRING(16383)
   RETURNS CSTRING(16383)
  ENTRY_POINT 'upper'  MODULE_NAME 'FreeAdhocUDF';

DECLARE EXTERNAL FUNCTION RLOWER
   CSTRING(255)
   RETURNS CSTRING(255)
  ENTRY_POINT 'lower'  MODULE_NAME 'FreeAdhocUDF';

DECLARE EXTERNAL FUNCTION LONGRLOWER
   CSTRING(16383)
   RETURNS CSTRING(16383)
  ENTRY_POINT 'lower'  MODULE_NAME 'FreeAdhocUDF';

DECLARE EXTERNAL FUNCTION STRPOS
   CSTRING(16383), CSTRING(16383)
   RETURNS INTEGER FREE_IT
  ENTRY_POINT 'strpos'  MODULE_NAME 'FreeAdhocUDF';

DECLARE EXTERNAL FUNCTION STRCOUNT
   CSTRING(255), CSTRING(16383)
   RETURNS INTEGER FREE_IT
  ENTRY_POINT 'r_stringcount'  MODULE_NAME 'FreeAdhocUDF';

DECLARE EXTERNAL FUNCTION STRCMP
   CSTRING(16383), CSTRING(16383)
   RETURNS SMALLINT FREE_IT
  ENTRY_POINT 'r_strcmp'  MODULE_NAME 'FreeAdhocUDF';

DECLARE EXTERNAL FUNCTION WORDCOUNT
   CSTRING(16383),
   CSTRING(31),
   SMALLINT
   RETURNS INTEGER FREE_IT
  ENTRY_POINT 'r_wordcount'  MODULE_NAME 'FreeAdhocUDF';

DECLARE EXTERNAL FUNCTION WORDNUM
   CSTRING(16383),
   INTEGER,
   CSTRING(31),
   SMALLINT
   RETURNS CSTRING(255)
  ENTRY_POINT 'wordnum'  MODULE_NAME 'FreeAdhocUDF';

DECLARE EXTERNAL FUNCTION LONGWORDNUM
   CSTRING(16383),
   INTEGER,
   CSTRING(31),
   SMALLINT
   RETURNS CSTRING(16383)
  ENTRY_POINT 'wordnum'  MODULE_NAME 'FreeAdhocUDF';

DECLARE EXTERNAL FUNCTION PADRIGHT
   CSTRING(255), SMALLINT, CSTRING(1)
   RETURNS CSTRING(255) FREE_IT
  ENTRY_POINT 'r_padright'  MODULE_NAME 'FreeAdhocUDF';

DECLARE EXTERNAL FUNCTION LONGPADRIGHT
   CSTRING(16383), SMALLINT, CSTRING(1)
   RETURNS CSTRING(16383) FREE_IT
  ENTRY_POINT 'r_padright'  MODULE_NAME 'FreeAdhocUDF';

DECLARE EXTERNAL FUNCTION PADLEFT
   CSTRING(255), SMALLINT, CSTRING(1)
   RETURNS CSTRING(255) FREE_IT
  ENTRY_POINT 'r_padleft'  MODULE_NAME 'FreeAdhocUDF';

DECLARE EXTERNAL FUNCTION LONGPADLEFT
   CSTRING(16383), SMALLINT, CSTRING(1)
   RETURNS CSTRING(16383) FREE_IT
  ENTRY_POINT 'r_padleft'  MODULE_NAME 'FreeAdhocUDF';

DECLARE EXTERNAL FUNCTION C
   CSTRING(255)
   RETURNS CSTRING(255)
  ENTRY_POINT 'r_c'  MODULE_NAME 'FreeAdhocUDF';

DECLARE EXTERNAL FUNCTION LONGC
   CSTRING(16383)
   RETURNS CSTRING(16383)
  ENTRY_POINT 'r_c'  MODULE_NAME 'FreeAdhocUDF';

DECLARE EXTERNAL FUNCTION REPEATTRIM
   CSTRING(255), CSTRING(255)
   RETURNS CSTRING(255)
  ENTRY_POINT 'repeattrim'  MODULE_NAME 'FreeAdhocUDF';

DECLARE EXTERNAL FUNCTION LONGREPEATTRIM
   CSTRING(16383), CSTRING(16383)
   RETURNS CSTRING(16383)
  ENTRY_POINT 'repeattrim'  MODULE_NAME 'FreeAdhocUDF';

DECLARE EXTERNAL FUNCTION STRREPEAT
   CSTRING(255), INTEGER
   RETURNS CSTRING(255) FREE_IT
  ENTRY_POINT 'strrepeat'  MODULE_NAME 'FreeAdhocUDF';

DECLARE EXTERNAL FUNCTION LONGSTRREPEAT
   CSTRING(16383), INTEGER
   RETURNS CSTRING(16383) FREE_IT
  ENTRY_POINT 'strrepeat'  MODULE_NAME 'FreeAdhocUDF';

DECLARE EXTERNAL FUNCTION STRSTUFF
   CSTRING(255), INTEGER, INTEGER, CSTRING(255)
   RETURNS CSTRING(255) FREE_IT
  ENTRY_POINT 'strstuff'  MODULE_NAME 'FreeAdhocUDF';

DECLARE EXTERNAL FUNCTION LONGSTRSTUFF
   CSTRING(16383), INTEGER, INTEGER, CSTRING(16383)
   RETURNS CSTRING(16383) FREE_IT
  ENTRY_POINT 'longstrstuff'  MODULE_NAME 'FreeAdhocUDF';

DECLARE EXTERNAL FUNCTION FLOATTOSTR
   DOUBLE PRECISION, CSTRING(255)
   RETURNS CSTRING(255) FREE_IT
  ENTRY_POINT 'floattostr'  MODULE_NAME 'FreeAdhocUDF';

DECLARE EXTERNAL FUNCTION INTTOSTR
   INTEGER, CSTRING(255)
   RETURNS CSTRING(255) FREE_IT
  ENTRY_POINT 'inttostr'  MODULE_NAME 'FreeAdhocUDF';

/* Win1251 functions */

DECLARE EXTERNAL FUNCTION RUPPER
   CSTRING(255)
   RETURNS CSTRING(255) FREE_IT
  ENTRY_POINT 'rupper'  MODULE_NAME 'FreeAdhocUDF';

DECLARE EXTERNAL FUNCTION LONGRUPPER
   CSTRING(16383)
   RETURNS CSTRING(16383) FREE_IT
  ENTRY_POINT 'rupper'  MODULE_NAME 'FreeAdhocUDF';

DECLARE EXTERNAL FUNCTION RLOWER
   CSTRING(255)
   RETURNS CSTRING(255) FREE_IT
  ENTRY_POINT 'rlower'  MODULE_NAME 'FreeAdhocUDF';

DECLARE EXTERNAL FUNCTION LONGRLOWER
   CSTRING(16383)
   RETURNS CSTRING(16383) FREE_IT
  ENTRY_POINT 'rlower'  MODULE_NAME 'FreeAdhocUDF';

DECLARE EXTERNAL FUNCTION RLATIN
   CSTRING(255)
   RETURNS CSTRING(255) FREE_IT
  ENTRY_POINT 'rlatin'  MODULE_NAME 'FreeAdhocUDF';

DECLARE EXTERNAL FUNCTION LONGRLATIN
   CSTRING(16383)
   RETURNS CSTRING(16383) FREE_IT
  ENTRY_POINT 'rlatin'  MODULE_NAME 'FreeAdhocUDF';

/* not yet

DECLARE EXTERNAL FUNCTION RTRANSLIT
   CSTRING(255)
   RETURNS CSTRING(255) FREE_IT
  ENTRY_POINT 'fn_rtranslit'  MODULE_NAME 'FreeAdhocUDF';

DECLARE EXTERNAL FUNCTION LONGRTRANSLIT
   CSTRING(16383)
   RETURNS CSTRING(16383) FREE_IT
  ENTRY_POINT 'fn_longrtranslit'  MODULE_NAME 'FreeAdhocUDF';

DECLARE EXTERNAL FUNCTION EAN13CS
   CSTRING(255)
   RETURNS SMALLINT FREE_IT
  ENTRY_POINT 'fn_ean13cs'  MODULE_NAME 'FreeAdhocUDF';

*/

/* never done because only use with russian character set

DECLARE EXTERNAL FUNCTION NUMINWORDS
   INTEGER, CSTRING(1)
   RETURNS CSTRING(255) FREE_IT
  ENTRY_POINT 'fn_numinwords'  MODULE_NAME 'FreeAdhocUDF';

DECLARE EXTERNAL FUNCTION CONVERTSYMBOLS
   CSTRING(255), CSTRING(255), CSTRING(255)
   RETURNS CSTRING(255)
  ENTRY_POINT 'fn_convertsymbols'  MODULE_NAME 'FreeAdhocUDF';

DECLARE EXTERNAL FUNCTION LONGCONVERTSYMBOLS
   CSTRING(16383), CSTRING(255), CSTRING(255)
   RETURNS CSTRING(16383)
  ENTRY_POINT 'fn_convertsymbols'  MODULE_NAME 'FreeAdhocUDF';

*/

/* not yet NVL-functions

DECLARE EXTERNAL FUNCTION IIF
   INTEGER, INTEGER, INTEGER
   RETURNS INTEGER FREE_IT
  ENTRY_POINT 'fn_iif'  MODULE_NAME 'FreeAdhocUDF';

DECLARE EXTERNAL FUNCTION DIF
   INTEGER, DOUBLE PRECISION, DOUBLE PRECISION
   RETURNS DOUBLE PRECISION FREE_IT
  ENTRY_POINT 'fn_dif'  MODULE_NAME 'FreeAdhocUDF';

DECLARE EXTERNAL FUNCTION CIF
   INTEGER, CSTRING(255), CSTRING(255)
   RETURNS CSTRING(255)
  ENTRY_POINT 'fn_cif'  MODULE_NAME 'FreeAdhocUDF';

DECLARE EXTERNAL FUNCTION LONGCIF
   INTEGER, CSTRING(16383), CSTRING(16383)
   RETURNS CSTRING(16383)
  ENTRY_POINT 'fn_cif'  MODULE_NAME 'FreeAdhocUDF';

DECLARE EXTERNAL FUNCTION DTIF
   INTEGER, TIMESTAMP, TIMESTAMP
   RETURNS TIMESTAMP
  ENTRY_POINT 'fn_dtif'  MODULE_NAME 'FreeAdhocUDF';
*/

/* never done because not portable to Linux

DECLARE EXTERNAL FUNCTION MSGBOX
   CSTRING(16383),
   CSTRING(255),
   INTEGER
   RETURNS INTEGER FREE_IT
  ENTRY_POINT 'fn_msgbox'  MODULE_NAME 'FreeAdhocUDF';
*/

/******************************************************************************/
/****                User Defined Functions: UUID Functions                ****/
/****       to use ONLY with FireBird 2.x - possibility to return NULL     ****/
/******************************************************************************/

DECLARE EXTERNAL FUNCTION CREATEGUID
   RETURNS CSTRING(38) FREE_IT
  ENTRY_POINT 'uuid4'  MODULE_NAME 'FreeAdhocUDF';

/* never done
DECLARE EXTERNAL FUNCTION ADD_USER
   INTEGER, CSTRING(255),
   CSTRING(31), CSTRING(8),
   CSTRING(31), CSTRING(31), CSTRING(31), CSTRING(31),
   CSTRING(31), CSTRING(8)
   RETURNS INTEGER FREE_IT
  ENTRY_POINT 'fn_add_user'  MODULE_NAME 'FreeAdhocUDF';

DECLARE EXTERNAL FUNCTION MODIFY_USER
   INTEGER, CSTRING(255),
   CSTRING(31), CSTRING(8),
   CSTRING(31), CSTRING(31), CSTRING(31), CSTRING(31),
   CSTRING(31), CSTRING(8)
   RETURNS INTEGER FREE_IT
  ENTRY_POINT 'fn_modify_user'  MODULE_NAME 'FreeAdhocUDF';

DECLARE EXTERNAL FUNCTION DELETE_USER
   INTEGER, CSTRING(255),
   CSTRING(31),
   CSTRING(31), CSTRING(8)
   RETURNS INTEGER FREE_IT
  ENTRY_POINT 'fn_delete_user'  MODULE_NAME 'FreeAdhocUDF';
*/

/******************************************************************************/
/****                   User Defined Functions - other functions           ****/
/******************************************************************************/

/* maybe later

DECLARE EXTERNAL FUNCTION CALCEXPR
   CSTRING(16383),
   CSTRING(16383)
   RETURNS DOUBLE PRECISION FREE_IT
  ENTRY_POINT 'fn_CalcExpr'  MODULE_NAME 'FreeAdhocUDF';

DECLARE EXTERNAL FUNCTION EXPRISVALID
   CSTRING(16383),
   CSTRING(16383)
   RETURNS INTEGER FREE_IT
  ENTRY_POINT 'fn_ExprIsValid'  MODULE_NAME 'FreeAdhocUDF';

*/

/******************************************************************************/
/****                User Defined Functions - BLOb Functions               ****/
/****            to use with FireBird 2.x - possibility to return NULL     ****/
/******************************************************************************/

DECLARE EXTERNAL FUNCTION B_NUMBER_SEGMENTS
   BLOB
   RETURNS INTEGER FREE_IT
  ENTRY_POINT 'blobsegmentcount'  MODULE_NAME 'FreeAdhocUDF';

DECLARE EXTERNAL FUNCTION B_MAX_SEGMENT
   BLOB
   RETURNS INTEGER FREE_IT
  ENTRY_POINT 'blobmaxsegmentlength'  MODULE_NAME 'FreeAdhocUDF';

DECLARE EXTERNAL FUNCTION B_TOTAL_LENGTH
   BLOB
   RETURNS INTEGER FREE_IT
  ENTRY_POINT 'blobsize'  MODULE_NAME 'FreeAdhocUDF';

DECLARE EXTERNAL FUNCTION B_LINE_COUNT
   BLOB
   RETURNS INTEGER FREE_IT
  ENTRY_POINT 'bloblinecount'  MODULE_NAME 'FreeAdhocUDF';

DECLARE EXTERNAL FUNCTION B_SUBSTR
   BLOB, INTEGER, INTEGER
   RETURNS CSTRING(255) FREE_IT
  ENTRY_POINT 'r_blobmid'  MODULE_NAME 'FreeAdhocUDF';

DECLARE EXTERNAL FUNCTION B_LONGSUBSTR
   BLOB, INTEGER, INTEGER
   RETURNS CSTRING(16383) FREE_IT
  ENTRY_POINT 'r_blobmid'  MODULE_NAME 'FreeAdhocUDF';

DECLARE EXTERNAL FUNCTION B_LINE
   BLOB, INTEGER
   RETURNS CSTRING(255) FREE_IT
  ENTRY_POINT 'blobline'  MODULE_NAME 'FreeAdhocUDF';

DECLARE EXTERNAL FUNCTION B_LONGLINE
   BLOB, INTEGER
   RETURNS CSTRING(16383) FREE_IT
  ENTRY_POINT 'blobline'  MODULE_NAME 'FreeAdhocUDF';

DECLARE EXTERNAL FUNCTION B_PUT_SEGMENT
   CSTRING(16383),
   BLOB
   RETURNS PARAMETER 2
  ENTRY_POINT 'strblob'  MODULE_NAME 'FreeAdhocUDF';

DECLARE EXTERNAL FUNCTION B_STRPOS
   CSTRING(16383),
   BLOB
   RETURNS INTEGER FREE_IT
  ENTRY_POINT 'b_strpos'  MODULE_NAME 'FreeAdhocUDF';

DECLARE EXTERNAL FUNCTION B_STRCMP
   BLOB, BLOB
   RETURNS SMALLINT FREE_IT
  ENTRY_POINT 'b_strcmp'  MODULE_NAME 'FreeAdhocUDF';

DECLARE EXTERNAL FUNCTION B_TEXTPOS
   CSTRING(16383),
   BLOB
   RETURNS INTEGER FREE_IT
  ENTRY_POINT 'b_textpos'  MODULE_NAME 'FreeAdhocUDF';

/* Win1251 functions
   never done because only use with russion character set

DECLARE EXTERNAL FUNCTION RLATIN
   CSTRING(255)
   RETURNS CSTRING(255)
  ENTRY_POINT 'fn_rlatin'  MODULE_NAME 'FreeAdhocUDF';

DECLARE EXTERNAL FUNCTION LONGRLATIN
   CSTRING(16383)
   RETURNS CSTRING(16383)
  ENTRY_POINT 'fn_rlatin'  MODULE_NAME 'FreeAdhocUDF';

DECLARE EXTERNAL FUNCTION RTRANSLIT
   CSTRING(255)
   RETURNS CSTRING(255) FREE_IT
  ENTRY_POINT 'fn_rtranslit'  MODULE_NAME 'FreeAdhocUDF';

DECLARE EXTERNAL FUNCTION LONGRTRANSLIT
   CSTRING(16383)
   RETURNS CSTRING(16383) FREE_IT
  ENTRY_POINT 'fn_longrtranslit'  MODULE_NAME 'FreeAdhocUDF';

*/

/* File functions
   never done because not portable to Linux

DECLARE EXTERNAL FUNCTION FCREATE
   CSTRING(255), INTEGER
   RETURNS INTEGER FREE_IT
  ENTRY_POINT 'fn_fcreate' MODULE_NAME 'FreeAdhocUDF';

DECLARE EXTERNAL FUNCTION FOPEN
   CSTRING(255), INTEGER, INTEGER
   RETURNS INTEGER FREE_IT
  ENTRY_POINT 'fn_fopen' MODULE_NAME 'FreeAdhocUDF';

DECLARE EXTERNAL FUNCTION FREAD
   INTEGER, INTEGER
   RETURNS CSTRING(255) FREE_IT
  ENTRY_POINT 'fn_fread' MODULE_NAME 'FreeAdhocUDF';

DECLARE EXTERNAL FUNCTION LONGFREAD
   INTEGER, INTEGER
   RETURNS CSTRING(16383) FREE_IT
  ENTRY_POINT 'fn_fread' MODULE_NAME 'FreeAdhocUDF';

DECLARE EXTERNAL FUNCTION FWRITE
   INTEGER, CSTRING(16383)
   RETURNS INTEGER FREE_IT
  ENTRY_POINT 'fn_fwrite' MODULE_NAME 'FreeAdhocUDF';

DECLARE EXTERNAL FUNCTION FSEEK
   INTEGER, INTEGER, INTEGER
   RETURNS INTEGER FREE_IT
  ENTRY_POINT 'fn_fseek' MODULE_NAME 'FreeAdhocUDF';

DECLARE EXTERNAL FUNCTION FCLOSE
   INTEGER
   RETURNS INTEGER FREE_IT
  ENTRY_POINT 'fn_fclose' MODULE_NAME 'FreeAdhocUDF';

DECLARE EXTERNAL FUNCTION FREMOVE
   CSTRING(255)
   RETURNS INTEGER FREE_IT
  ENTRY_POINT 'fn_fremove' MODULE_NAME 'FreeAdhocUDF';

DECLARE EXTERNAL FUNCTION FSIZE
   CSTRING(255)
   RETURNS INTEGER FREE_IT
  ENTRY_POINT 'fn_fsize' MODULE_NAME 'FreeAdhocUDF';

 File bit mask constants

DECLARE EXTERNAL FUNCTION FSEEK_CUR
   RETURNS INTEGER FREE_IT
  ENTRY_POINT 'fn_fseek_cur' MODULE_NAME 'FreeAdhocUDF';

DECLARE EXTERNAL FUNCTION FSEEK_END
   RETURNS INTEGER FREE_IT
  ENTRY_POINT 'fn_fseek_end' MODULE_NAME 'FreeAdhocUDF';

DECLARE EXTERNAL FUNCTION FSEEK_SET
   RETURNS INTEGER FREE_IT
  ENTRY_POINT 'fn_fseek_set' MODULE_NAME 'FreeAdhocUDF';

DECLARE EXTERNAL FUNCTION FO_RDONLY
   RETURNS INTEGER FREE_IT
  ENTRY_POINT 'fn_fo_rdonly' MODULE_NAME 'FreeAdhocUDF';

DECLARE EXTERNAL FUNCTION FO_WRONLY
   RETURNS INTEGER FREE_IT
  ENTRY_POINT 'fn_fo_wronly' MODULE_NAME 'FreeAdhocUDF';

DECLARE EXTERNAL FUNCTION FO_RDWR
   RETURNS INTEGER FREE_IT
  ENTRY_POINT 'fn_fo_rdwr' MODULE_NAME 'FreeAdhocUDF';

DECLARE EXTERNAL FUNCTION FO_CREAT
   RETURNS INTEGER FREE_IT
  ENTRY_POINT 'fn_fo_creat' MODULE_NAME 'FreeAdhocUDF';

DECLARE EXTERNAL FUNCTION FO_TRUNC
   RETURNS INTEGER FREE_IT
  ENTRY_POINT 'fn_fo_trunc' MODULE_NAME 'FreeAdhocUDF';

DECLARE EXTERNAL FUNCTION FO_EXCL
   RETURNS INTEGER FREE_IT
  ENTRY_POINT 'fn_fo_excl' MODULE_NAME 'FreeAdhocUDF';

DECLARE EXTERNAL FUNCTION FO_APPEND
   RETURNS INTEGER FREE_IT
  ENTRY_POINT 'fn_fo_append' MODULE_NAME 'FreeAdhocUDF';

DECLARE EXTERNAL FUNCTION FS_IREAD
   RETURNS INTEGER FREE_IT
  ENTRY_POINT 'fn_fs_iread' MODULE_NAME 'FreeAdhocUDF';

DECLARE EXTERNAL FUNCTION FS_IWRITE
   RETURNS INTEGER FREE_IT
  ENTRY_POINT 'fn_fs_iwrite' MODULE_NAME 'FreeAdhocUDF';

DECLARE EXTERNAL FUNCTION FS_IEXEC
   RETURNS INTEGER FREE_IT
  ENTRY_POINT 'fn_fs_iexec' MODULE_NAME 'FreeAdhocUDF';

*/

/* MD5 checksum functions
   not yet

DECLARE EXTERNAL FUNCTION MD5SUM
   CSTRING(16383)
   RETURNS CSTRING(32) FREE_IT
  ENTRY_POINT 'fn_md5sum'  MODULE_NAME 'FreeAdhocUDF';

DECLARE EXTERNAL FUNCTION MD5INIT
   RETURNS CSTRING(176) FREE_IT
  ENTRY_POINT 'fn_md5init'  MODULE_NAME 'FreeAdhocUDF';

DECLARE EXTERNAL FUNCTION MD5UPDATE
   CSTRING(176), CSTRING(16383), INTEGER
   RETURNS CSTRING(176)
  ENTRY_POINT 'fn_md5update'  MODULE_NAME 'FreeAdhocUDF';

DECLARE EXTERNAL FUNCTION MD5FINAL
   CSTRING(176)
   RETURNS CSTRING(32) FREE_IT
  ENTRY_POINT 'fn_md5final'  MODULE_NAME 'FreeAdhocUDF';

*/

